==> v0.1.0.zip: clickguard/tools/lib/xpi-builder.sh <==
# -*- coding: utf-8; tab-width: 4 -*-
# vim: ts=4 noet ai

# Part of Zombie Navigator
# Copyright © 2016  Zombie Navigator Developers

xpiBuild () {
(
	versionInfoPath=$1
	srcDir=$2
	isDebugBuild=$3
	
	which advzip >/dev/null && zipper='advzip -a -4 -i 500' || zipper=zip
	
	# Normalize build environment
	unset LANGUAGE DISPLAY USER
	LANG=C
	LC_ALL=C
	
	# Load build information
	. "$versionInfoPath"
	
	# Create temporary directories
	[ -d /dev/shm ] && [ -w /dev/shm ] && [ -x /dev/shm ] \
		&& tmpBaseDir=/dev/shm || tmpBaseDir=/tmp
	
	tmpDir=`mktemp -d "$tmpBaseDir"/zombie-navigator-builder.XXXXXX`
	[ -d "$tmpDir" ] || {
		error "Failed to create a temporary directory"
		exit 1
	}
	
	tmpHomeDir="$tmpDir/home"
	tmpTmpDir="$tmpDir/tmp"
	mkdir "$tmpHomeDir" "$tmpTmpDir" >&2
	
	TMP="$tmpTmpDir"
	TMPDIR="$tmpTmpDir"
	TEMP="$tmpTmpDir"
	TEMPDIR="$tmpTmpDir"
	HOME="$tmpHomeDir"
	export TMP TMPDIR TEMP TEMPDIR HOME
	
	# Create build directory
	buildDir="$tmpDir/build"
	mkdir "$buildDir" >&2
	cp -r "$srcDir"/* "$buildDir/" >&2
	
	[ "$isDebugBuild" ] || {
		# Remove unnecessary files
		log "Excluding development files..."
		rm -frv "$buildDir/debug.js" "$buildDir/data/debug" >&2
		
		# Prepare files for packaging
		preprocess_egrep_v='^\s*/\*!_DEBUGONLY_!\*/'
		find "$buildDir" -name \*.js -exec \
			sh -c 'egrep -v "$1" < "$2" > "$2.tmp" && mv -f "$2.tmp" "$2"' \
			-- "$preprocess_egrep_v" {} \; >&2
		
		xpiOptimizeDir "$buildDir" >&2
	}
	
	# Create a package
	cd "$buildDir" >&2
	
	log "Generating the necessary files..."
	cat ./build.meta >> ./index.js
	version=`cat ./build.version`
	rm -fv ./build.meta ./build.version >&2
	
	id="@${addon_id}"
	name=$addon_name
	description=$addon_description
	developers=$addon_author
	genIndexRDF "$id" "$version" "$name" "$description" "$developers" \
		> install.rdf
	cp -v "$srcDir/../deps/jpm-core/data/bootstrap.js" ./ >&2
	
	log "Building the addon..."
	# Skipping directory entries
	# Assuming no space characters in file names
	$zipper ../build.zip `find . -type f` | progressFilter >&2
	cd .. >&2
	
	[ "$isDebugBuild" ] || {
		# Postprocessing
		which strip-nondeterminism >/dev/null && {
			log "Trying to strip nondeterminism (not yet tested)..."
			strip-nondeterminism -t zip ./build.zip >&2
		} || {
			warn "NOTE: Deterministic build disabled (strip-nondeterminism needed)..."
		}
		
		notice "SHA2-256: `sha256 < ./build.zip`"
	}
	
	cat ./build.zip
	
	# Clean up temporary files
	log "Cleaning up temporary files..."
	cd "$srcDir/.." >&2
	rm -frv "$tmpDir" | progressFilter >&2
	
	# Print filename
	printf %s "${id}-${version}" >&3
)
}

==> v0.1.2.zip: clickguard/tools/lib/xpi-builder.sh <==
# -*- coding: utf-8; tab-width: 4 -*-
# vim: ts=4 noet ai

# Part of Zombie Navigator
# Copyright © 2016  Zombie Navigator Developers

xpiBuild () {
(
	versionInfoPath=$1
	srcDir=$2
	isDebugBuild=$3
	
	which advzip >/dev/null && zipper='advzip -a -4 -i 500' || zipper=zip
	
	# Normalize build environment
	unset LANGUAGE DISPLAY USER
	LANG=C
	LC_ALL=C
	
	# Load build information
	. "$versionInfoPath"
	
	# Create temporary directories
	[ -d /dev/shm ] && [ -w /dev/shm ] && [ -x /dev/shm ] \
		&& tmpBaseDir=/dev/shm || tmpBaseDir=/tmp
	
	tmpDir=`mktemp -d "$tmpBaseDir"/zombie-navigator-builder.XXXXXX`
	[ -d "$tmpDir" ] || {
		error "Failed to create a temporary directory"
		exit 1
	}
	
	tmpHomeDir="$tmpDir/home"
	tmpTmpDir="$tmpDir/tmp"
	mkdir "$tmpHomeDir" "$tmpTmpDir" >&2
	
	TMP="$tmpTmpDir"
	TMPDIR="$tmpTmpDir"
	TEMP="$tmpTmpDir"
	TEMPDIR="$tmpTmpDir"
	HOME="$tmpHomeDir"
	export TMP TMPDIR TEMP TEMPDIR HOME
	
	# Create build directory
	buildDir="$tmpDir/build"
	mkdir "$buildDir" >&2
	cp -r "$srcDir"/* "$buildDir/" >&2
	
	[ "$isDebugBuild" ] || {
		# Remove unnecessary files
		log "Excluding development files..."
		rm -frv "$buildDir/debug.js" "$buildDir/data/debug" >&2
		
		# Prepare files for packaging
		preprocess_egrep_v='^\s*/\*!_DEBUGONLY_!\*/'
		find "$buildDir" -name \*.js -exec \
			sh -c 'egrep -v "$1" < "$2" > "$2.tmp" && mv -f "$2.tmp" "$2"' \
			-- "$preprocess_egrep_v" {} \; >&2
		
		xpiOptimizeDir "$buildDir" >&2
	}
	
	# Create a package
	cd "$buildDir" >&2
	
	log "Generating the necessary files..."
	cat ./build.meta >> ./main.js
	version=`cat ./build.version`
	rm -fv ./build.meta ./build.version >&2
	
	id="@${addon_id}"
	name=$addon_name
	description=$addon_description
	developers=$addon_author
	genIndexRDF "$id" "$version" "$name" "$description" "$developers" \
		> install.rdf
	cp -v "$srcDir/../deps/jpm-core/data/bootstrap.js" ./ >&2
	
	log "Building the addon..."
	# Skipping directory entries
	# Assuming no space characters in file names
	$zipper ../build.zip `find . -type f` | progressFilter >&2
	cd .. >&2
	
	[ "$isDebugBuild" ] || {
		# Postprocessing
		which strip-nondeterminism >/dev/null && {
			log "Trying to strip nondeterminism (not yet tested)..."
			strip-nondeterminism -t zip ./build.zip >&2
		} || {
			warn "NOTE: Deterministic build disabled (strip-nondeterminism needed)..."
		}
		
		notice "SHA2-256: `sha256 < ./build.zip`"
	}
	
	cat ./build.zip
	
	# Clean up temporary files
	log "Cleaning up temporary files..."
	cd "$srcDir/.." >&2
	rm -frv "$tmpDir" | progressFilter >&2
	
	# Print filename
	printf %s "${id}-${version}" >&3
)
}
